@@ -45,6 +45,61 @@
 #include "multiplayer.h"
 #include "sound.h"
 
+static const int cheat_sequence[] =
+{
+    KEY_UPARROW, KEY_UPARROW, KEY_DOWNARROW, KEY_DOWNARROW,
+    KEY_LEFTARROW, KEY_RIGHTARROW, KEY_LEFTARROW, KEY_RIGHTARROW,
+    'b', 'a', KEY_ENTER, 0
+};
+
+static unsigned int cheat_sequence_index = 0;
+
+// I think these are good "sensible" defaults:
+
+static void SensibleDefaults(void)
+{
+    key_up = 'w';
+    key_down = 's';
+    key_strafeleft = 'a';
+    key_straferight = 'd';
+    mousebprevweapon = 4;
+    mousebnextweapon = 3;
+    snd_musicdevice = 3;
+    joybspeed = 29;
+    vanilla_savegame_limit = 0;
+    vanilla_keyboard_mapping = 0;
+    vanilla_demo_limit = 0;
+    show_endoom = 0;
+    dclick_use = 0;
+    novert = 1;
+}
+
+static int MainMenuKeyPress(txt_window_t *window, int key, void *user_data)
+{
+    if (key == cheat_sequence[cheat_sequence_index])
+    {
+        ++cheat_sequence_index;
+
+        if (cheat_sequence[cheat_sequence_index] == 0)
+        {
+            SensibleDefaults();
+            cheat_sequence_index = 0;
+
+            window = TXT_NewWindow(NULL);
+            TXT_AddWidget(window, TXT_NewLabel("    \x01    "));
+            TXT_SetWindowAction(window, TXT_HORIZ_RIGHT, NULL);
+
+            return 1;
+        }
+    }
+    else
+    {
+        cheat_sequence_index = 0;
+    }
+
+    return 0;
+}
+
 static void DoQuit(void *widget, void *dosave)
 {
     if (dosave != NULL)
@@ -97,7 +152,7 @@ static void LaunchDoom(void *unused1, void *unused2)
     // Launch Doom
 
     exec = NewExecuteContext();
-    AddConfigParameters(exec);
+    PassThroughArguments(exec);
     ExecuteDoom(exec);
 
     exit(0);
@@ -107,6 +162,7 @@ void MainMenu(void)
 {
     txt_window_t *window;
     txt_window_action_t *quit_action;
+    txt_window_action_t *warp_action;
 
     window = TXT_NewWindow("Main Menu");
 
@@ -134,8 +190,14 @@ void MainMenu(void)
           NULL);
 
     quit_action = TXT_NewWindowAction(KEY_ESCAPE, "Quit");
+    warp_action = TXT_NewWindowAction(KEY_F1, "Warp");
     TXT_SignalConnect(quit_action, "pressed", QuitConfirm, NULL);
+    TXT_SignalConnect(warp_action, "pressed",
+                      (TxtWidgetSignalFunc) WarpMenu, NULL);
     TXT_SetWindowAction(window, TXT_HORIZ_LEFT, quit_action);
+    TXT_SetWindowAction(window, TXT_HORIZ_CENTER, warp_action);
+
+    TXT_SetKeyListener(window, MainMenuKeyPress, NULL);
 }
 
 //
@@ -194,11 +256,9 @@ static void SetIcon(void)
     free(mask);
 }
 
-// 
-// Initialize and run the textscreen GUI.
-//
+// Initialize the textscreen library.
 
-static void RunGUI(void)
+static void InitTextscreen(void)
 {
     SetDisplayDriver();
 
@@ -210,7 +270,24 @@ static void RunGUI(void)
 
     TXT_SetDesktopTitle(PACKAGE_NAME " Setup ver " PACKAGE_VERSION);
     SetIcon();
-    
+}
+
+// Restart the textscreen library.  Used when the video_driver variable
+// is changed.
+
+void RestartTextscreen(void)
+{
+    TXT_Shutdown();
+    InitTextscreen();
+}
+
+// 
+// Initialize and run the textscreen GUI.
+//
+
+static void RunGUI(void)
+{
+    InitTextscreen();
     MainMenu();
 
     TXT_GUIMainLoop();
