@@ -735,10 +735,12 @@ static void InitGameVersion(void)
             else
             {
                 // Final Doom: tnt or plutonia
-                // Default to the "alt" version of the executable that
-                // fixes the demo loop behavior.
+                // Defaults to emulating the first Final Doom executable,
+                // which has the crash in the demo loop; however, having
+                // this as the default should mean that it plays back
+                // most demos correctly.
 
-                gameversion = exe_final2;
+                gameversion = exe_final;
             }
         }
     }
@@ -890,9 +892,7 @@ void D_DoomMain (void)
 
     if (M_CheckParm("-search"))
     {
-        printf("\nSearching for servers on Internet ...\n");
-        p = NET_MasterQuery(NET_QueryPrintCallback, NULL);
-        printf("\n%i server(s) found.\n", p);
+        NET_MasterQuery();
         exit(0);
     }
 
@@ -920,9 +920,7 @@ void D_DoomMain (void)
 
     if (M_CheckParm("-localsearch"))
     {
-        printf("\nSearching for servers on local LAN ...\n");
-        p = NET_LANQuery(NET_QueryPrintCallback, NULL);
-        printf("\n%i server(s) found.\n", p);
+        NET_LANQuery();
         exit(0);
     }
 
