@@ -85,6 +85,7 @@ static int num_targets;
 
 static boolean query_loop_running = false;
 static boolean printed_header = false;
+static int last_query_time = 0;
 
 // Resolve the master server address.
 
@@ -242,10 +243,35 @@ static void NET_Query_ParseResponse(net_addr_t *addr, net_packet_t *packet,
         return;
     }
 
-    // Find the target that responded, or potentially add a new target
-    // if it was not already known (for LAN broadcast search)
+    // Find the target that responded.
 
-    target = GetTargetForAddr(addr, true);
+    target = GetTargetForAddr(addr, false);
+
+    // If the target is not found, it may be because we are doing
+    // a LAN broadcast search, in which case we need to create a
+    // target for the new responder.
+
+    if (target == NULL)
+    {
+        query_target_t *broadcast_target;
+
+        broadcast_target = GetTargetForAddr(NULL, false);
+
+        // Not in broadcast mode, unexpected response that came out
+        // of nowhere. Ignore.
+
+        if (broadcast_target == NULL
+         || broadcast_target->state != QUERY_TARGET_QUERIED)
+        {
+            return;
+        }
+
+        // Create new target.
+
+        target = GetTargetForAddr(addr, true);
+        target->state = QUERY_TARGET_QUERIED;
+        target->query_time = broadcast_target->query_time;
+    }
 
     if (target->state != QUERY_TARGET_RESPONDED)
     {
@@ -351,6 +377,13 @@ static void SendOneQuery(void)
 
     now = I_GetTimeMS();
 
+    // Rate limit - only send one query every 50ms.
+
+    if (now - last_query_time < 50)
+    {
+        return;
+    }
+
     for (i = 0; i < num_targets; ++i)
     {
         // Not queried yet?
@@ -389,8 +422,10 @@ static void SendOneQuery(void)
 
     //printf("Queried %s\n", NET_AddrToString(targets[i].addr));
     targets[i].state = QUERY_TARGET_QUERIED;
-    targets[i].query_time = I_GetTimeMS();
+    targets[i].query_time = now;
     ++targets[i].query_attempts;
+
+    last_query_time = now;
 }
 
 // Time out servers that have been queried and not responded.
@@ -404,6 +439,12 @@ static void CheckTargetTimeouts(void)
 
     for (i = 0; i < num_targets; ++i)
     {
+        /*
+        printf("target %i: state %i, queries %i, query time %i\n",
+               i, targets[i].state, targets[i].query_attempts,
+               now - targets[i].query_time);
+        */
+
         // We declare a target to be "no response" when we've sent
         // multiple query packets to it (QUERY_MAX_ATTEMPTS) and
         // received no response to any of them.
@@ -413,6 +454,12 @@ static void CheckTargetTimeouts(void)
          && now - targets[i].query_time > QUERY_TIMEOUT_SECS * 1000)
         {
             targets[i].state = QUERY_TARGET_NO_RESPONSE;
+
+            if (targets[i].type == QUERY_TARGET_MASTER)
+            {
+                fprintf(stderr, "NET_MasterQuery: no response "
+                                "from master server.\n");
+            }
         }
     }
 }
@@ -435,6 +482,26 @@ static boolean AllTargetsDone(void)
     return true;
 }
 
+// Polling function, invoked periodically to send queries and
+// interpret new responses received from remote servers.
+// Returns zero when the query sequence has completed and all targets
+// have returned responses or timed out.
+
+int NET_Query_Poll(net_query_callback_t callback, void *user_data)
+{
+    CheckTargetTimeouts();
+
+    // Send a query.  This will only send a single query at once.
+
+    SendOneQuery();
+
+    // Check for a response
+
+    NET_Query_GetResponse(callback, user_data);
+
+    return !AllTargetsDone();
+}
+
 // Stop the query loop
 
 static void NET_Query_ExitLoop(void)
@@ -445,27 +512,15 @@ static void NET_Query_ExitLoop(void)
 // Loop waiting for responses.
 // The specified callback is invoked when a new server responds.
 
-static void NET_Query_QueryLoop(net_query_callback_t callback,
-                                void *user_data)
+static void NET_Query_QueryLoop(net_query_callback_t callback, void *user_data)
 {
     query_loop_running = true;
 
-    while (query_loop_running && !AllTargetsDone())
+    while (query_loop_running && NET_Query_Poll(callback, user_data))
     {
-        // Send a query.  This will only send a single query.
-        // Because of the delay below, this is therefore rate limited.
-
-        SendOneQuery();
-
-        // Check for a response
-
-        NET_Query_GetResponse(callback, user_data);
-
         // Don't thrash the CPU
 
-        I_Sleep(50);
-
-        CheckTargetTimeouts();
+        I_Sleep(1);
     }
 }
 
@@ -529,6 +584,155 @@ static int GetNumResponses(void)
     return result;
 }
 
+int NET_StartLANQuery(void)
+{
+    query_target_t *target;
+
+    NET_Query_Init();
+
+    // Add a broadcast target to the list.
+
+    target = GetTargetForAddr(NULL, true);
+    target->type = QUERY_TARGET_BROADCAST;
+
+    return 1;
+}
+
+int NET_StartMasterQuery(void)
+{
+    net_addr_t *master;
+    query_target_t *target;
+
+    NET_Query_Init();
+
+    // Resolve master address and add to targets list.
+
+    master = NET_Query_ResolveMaster(query_context);
+
+    if (master == NULL)
+    {
+        return 0;
+    }
+
+    target = GetTargetForAddr(master, true);
+    target->type = QUERY_TARGET_MASTER;
+
+    return 1;
+}
+
+// -----------------------------------------------------------------------
+
+static void formatted_printf(int wide, char *s, ...)
+{
+    va_list args;
+    int i;
+
+    va_start(args, s);
+    i = vprintf(s, args);
+    va_end(args);
+
+    while (i < wide)
+    {
+        putchar(' ');
+        ++i;
+    }
+}
+
+static char *GameDescription(GameMode_t mode, GameMission_t mission)
+{
+    switch (mode)
+    {
+        case shareware:
+            return "shareware";
+        case registered:
+            return "registered";
+        case retail:
+            return "ultimate";
+        case commercial:
+            if (mission == doom2)
+                return "doom2";
+            else if (mission == pack_tnt)
+                return "tnt";
+            else if (mission == pack_plut)
+                return "plutonia";
+        default:
+            return "unknown";
+    }
+}
+
+static void PrintHeader(void)
+{
+    int i;
+
+    putchar('\n');
+    formatted_printf(5, "Ping");
+    formatted_printf(18, "Address");
+    formatted_printf(8, "Players");
+    puts("Description");
+
+    for (i=0; i<70; ++i)
+        putchar('=');
+    putchar('\n');
+}
+
+// Callback function that just prints information in a table.
+
+static void NET_QueryPrintCallback(net_addr_t *addr,
+                                   net_querydata_t *data,
+                                   unsigned int ping_time,
+                                   void *user_data)
+{
+    // If this is the first server, print the header.
+
+    if (!printed_header)
+    {
+        PrintHeader();
+        printed_header = true;
+    }
+
+    formatted_printf(5, "%4i", ping_time);
+    formatted_printf(18, "%s: ", NET_AddrToString(addr));
+    formatted_printf(8, "%i/%i", data->num_players, 
+                                 data->max_players);
+
+    if (data->gamemode != indetermined)
+    {
+        printf("(%s) ", GameDescription(data->gamemode, 
+                                        data->gamemission));
+    }
+
+    if (data->server_state)
+    {
+        printf("(game running) ");
+    }
+
+    NET_SafePuts(data->description);
+}
+
+void NET_LANQuery(void)
+{
+    if (NET_StartLANQuery())
+    {
+        printf("\nSearching for servers on local LAN ...\n");
+
+        NET_Query_QueryLoop(NET_QueryPrintCallback, NULL);
+
+        printf("\n%i server(s) found.\n", GetNumResponses());
+    }
+}
+
+void NET_MasterQuery(void)
+{
+    if (NET_StartMasterQuery())
+    {
+        printf("\nSearching for servers on Internet ...\n");
+
+        NET_Query_QueryLoop(NET_QueryPrintCallback, NULL);
+
+        printf("\n%i server(s) found.\n", GetNumResponses());
+    }
+}
+
 void NET_QueryAddress(char *addr_str)
 {
     net_addr_t *addr;
@@ -593,138 +797,3 @@ net_addr_t *NET_FindLANServer(void)
     }
 }
 
-int NET_LANQuery(net_query_callback_t callback, void *user_data)
-{
-    query_target_t *target;
-
-    NET_Query_Init();
-
-    // Add a broadcast target to the list.
-
-    target = GetTargetForAddr(NULL, true);
-    target->type = QUERY_TARGET_BROADCAST;
-
-    NET_Query_QueryLoop(callback, user_data);
-
-    return GetNumResponses();
-}
-
-int NET_MasterQuery(net_query_callback_t callback, void *user_data)
-{
-    net_addr_t *master;
-    query_target_t *target;
-
-    NET_Query_Init();
-
-    // Resolve master address and add to targets list.
-
-    master = NET_Query_ResolveMaster(query_context);
-
-    if (master == NULL)
-    {
-        return 0;
-    }
-
-    target = GetTargetForAddr(master, true);
-    target->type = QUERY_TARGET_MASTER;
-
-    NET_Query_QueryLoop(callback, user_data);
-
-    // Check that we got a response from the master, and display
-    // a warning if we didn't.
-
-    if (target->state == QUERY_TARGET_NO_RESPONSE)
-    {
-        fprintf(stderr, "NET_MasterQuery: no response from master server.\n");
-    }
-
-    return GetNumResponses();
-}
-
-static void formatted_printf(int wide, char *s, ...)
-{
-    va_list args;
-    int i;
-
-    va_start(args, s);
-    i = vprintf(s, args);
-    va_end(args);
-
-    while (i < wide)
-    {
-        putchar(' ');
-        ++i;
-    }
-}
-
-static char *GameDescription(GameMode_t mode, GameMission_t mission)
-{
-    switch (mode)
-    {
-        case shareware:
-            return "shareware";
-        case registered:
-            return "registered";
-        case retail:
-            return "ultimate";
-        case commercial:
-            if (mission == doom2)
-                return "doom2";
-            else if (mission == pack_tnt)
-                return "tnt";
-            else if (mission == pack_plut)
-                return "plutonia";
-        default:
-            return "unknown";
-    }
-}
-
-static void PrintHeader(void)
-{
-    int i;
-
-    putchar('\n');
-    formatted_printf(5, "Ping");
-    formatted_printf(18, "Address");
-    formatted_printf(8, "Players");
-    puts("Description");
-
-    for (i=0; i<70; ++i)
-        putchar('=');
-    putchar('\n');
-}
-
-// Callback function that just prints information in a table.
-
-void NET_QueryPrintCallback(net_addr_t *addr,
-                            net_querydata_t *data,
-                            unsigned int ping_time,
-                            void *user_data)
-{
-    // If this is the first server, print the header.
-
-    if (!printed_header)
-    {
-        PrintHeader();
-        printed_header = true;
-    }
-
-    formatted_printf(5, "%4i", ping_time);
-    formatted_printf(18, "%s: ", NET_AddrToString(addr));
-    formatted_printf(8, "%i/%i", data->num_players, 
-                                 data->max_players);
-
-    if (data->gamemode != indetermined)
-    {
-        printf("(%s) ", GameDescription(data->gamemode, 
-                                        data->gamemission));
-    }
-
-    if (data->server_state)
-    {
-        printf("(game running) ");
-    }
-
-    NET_SafePuts(data->description);
-}
-
