@@ -170,7 +170,7 @@ static boolean NET_SDL_InitClient(void)
     // the default (2342).
     //
 
-    p = M_CheckParm("-port");
+    p = M_CheckParmWithArgs("-port", 1);
     if (p > 0)
         port = atoi(myargv[p+1]);
 
@@ -196,7 +196,7 @@ static boolean NET_SDL_InitServer(void)
 {
     int p;
     
-    p = M_CheckParm("-port");
+    p = M_CheckParmWithArgs("-port", 1);
     if (p > 0)
         port = atoi(myargv[p+1]);
 
